/* tslint:disable */
/* eslint-disable */
/**
* @param {any} value
* @returns {any}
*/
export function create_nested_attr_type_from_js(value: any): any;
/**
*/
export enum AttributeType {
  Boolean = 0,
  Binary = 1,
  Text = 2,
  Numeric = 3,
  DateTime = 4,
}
/**
*/
export enum Encoding {
  Base64 = 0,
  Utf8 = 1,
  Iso8859_1 = 2,
}

type IMeta = {
  [language: string]: {
    [attribute_name: string]: string
  }
}

type IAttribute = {
  name: string
  type: string
  is_flagged: boolean
  labels?: { [language: string]: string }
  category_labels?: { [language: string]: string }
  informations?: { [language: string]: string }
  entry_codes?: string[] | string
  entries?: { [language: string]: string | { [entry_code: string]: string } }
  mapping?: string
  encoding?: string
  format?: string
  units?: { [measurement_system: string]: string }
  entry_codes_mapping?: string[]
  reference_sai?: string
  condition?: string
  dependencies?: string[]
  cardinality?: string
  conformance?: 'O' | 'M'
  standards?: string[]
}



type Command = {
  type: 'Add',
  object_kind: string,
  content: {
    attributes?: { [attribute_name: string]: string },
    properties?: { [property_name: string]: string }
  }
}

type EntryCodeCommand = {
  type: 'Add',
  object_kind: 'EntryCode',
  content: {
    attributes?: {
      [attribute_name: string]: string | string[]
    },
    properties?: { [property_name: string]: string }
  }
}

type EntryCommand = {
  type: 'Add',
  object_kind: 'Entry',
  content: {
    attributes?: {
      [attribute_name: string]: string | {
        [entry_code: string]: string
      }
    },
    properties?: { [property_name: string]: string }
  }
}

type AST = {
  version: string,
  commands: (Command | EntryCodeCommand | EntryCommand)[],
  commands_meta?: object
}



type BundleWithDeps = {
  bundle: OCABundle,
  dependencies: string[]
}

type OCABundle = {
  v: string,
  d: string,
  capture_base: CaptureBase,
  overlays: Overlays,
  references?: {
    [capture_base_sai: string]: OCABundle
  }
}

type CaptureBase = {
  type: string,
  d: string,
  classification: string,
  attributes: { [attribute_name: string]: string },
  flagged_attributes: string[]
}

type Overlays = {
  cardinality?: CardinalityOverlay,
  character_encoding?: CharacterEncodingOverlay,
  conditional?: ConditionalOverlay,
  conformance?: ConformanceOverlay,
  entry?: EntryOverlay[],
  entry_code?: EntryCodeOverlay,
  entry_code_mapping?: EntryCodeMappingOverlay,
  format?: FormatOverlay,
  information?: InformationOverlay[],
  label?: LabelOverlay[],
  mapping?: MappingOverlay,
  meta?: MetaOverlay[],
  unit?: UnitOverlay[],
  standard?: StandardOverlay,
  subset?: SubsetOverlay,
  form_layout?: FormLayoutOverlay,
  credential_layout?: CredentialLayoutOverlay
}

type Overlay =
  | CardinalityOverlay
  | CharacterEncodingOverlay
  | ConditionalOverlay
  | ConformanceOverlay
  | EntryOverlay
  | EntryCodeOverlay
  | EntryCodeMappingOverlay
  | FormatOverlay
  | InformationOverlay
  | LabelOverlay
  | MappingOverlay
  | MetaOverlay
  | UnitOverlay
  | StandardOverlay
  | SubsetOverlay
  | FormLayoutOverlay
  | CredentialLayoutOverlay

type CardinalityOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attribute_cardinality: { [attribute_name: string]: string }
}

type CharacterEncodingOverlay = {
  capture_base: string,
  d: string,
  type: string,
  default_character_encoding: string,
  attribute_character_encoding: { [attribute_name: string]: string }
}

type ConditionalOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attribute_conditions: { [attribute_name: string]: string },
  attribute_dependencies: { [attribute_name: string]: string[] }
}

type ConformanceOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attribute_conformance: { [attribute_name: string]: 'O' | 'M' }
}

type CredentialLayoutOverlay = {
  capture_base: string,
  d: string,
  type: string,
  layout: {
    version: string,
    config?: {
      css?: {
        width?: string,
        height?: string,
        style?: string
      }
    },
    pages: {
      config?: {
        css?: {
          style?: string,
          classes?: string[],
          background_image?: string
        },
        name: string
      },
      elements: {
        type: string,
        size?: string,
        name?: string,
        layout?: string,
        content?: string,
        config?: {
          css?: {
            style?: string
            classes?: string[]
          }
        },
        elements?: CredentialLayoutOverlay['layout']['pages'][0]['elements']
      }[]
    }[],
    labels?: {
      [label: string]: {
        [language: string]: string
      }
    },
    reference_layouts?: {
      [reference_layout: string]: CredentialLayoutOverlay['layout']
    }
  }
}

type EntryOverlay = {
  capture_base: string,
  d: string,
  type: string,
  language: string,
  attribute_entries: { [attribute_name: string]: { [entry_code: string]: string } }
}

type EntryCodeOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attribute_entry_codes: { [attribute_name: string]: string[] }
}

type EntryCodeMappingOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attribute_entry_codes_mapping: { [attribute_name: string]: string[] }
}

type FormLayoutOverlay = {
  capture_base: string,
  d: string,
  type: string,
  layout: {
    config?: {
      css?: {
        style?: string
      }
    },
    elements: {
      type: string,
      config?: {
        css?: {
          style?: string,
          classes?: string[]
        }
      },
      id?: string,
      name?: string,
      parts?: {
        name: string,
        layout?: string,
        config?: {
          css?: {
            style?: string,
            classes?: string[]
          }
        }
      }[]
    }[],
    reference_layouts?: {
      [reference_layout: string]: FormLayoutOverlay['layout']
    }
  }
}

type FormatOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attribute_formats: { [attribute_name: string]: string }
}

type InformationOverlay = {
  capture_base: string,
  d: string,
  type: string,
  language: string,
  attribute_information: { [attribute_name: string]: string }
}

type LabelOverlay = {
  capture_base: string,
  d: string,
  type: string,
  language: string,
  attribute_labels: { [attribute_name: string]: string }
  attribute_categories: string[],
  category_labels: { [cat_id: string]: string },
  category_attributes: { [cat_id: string]: string[] }
}

type MappingOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attribute_mapping: { [attribute_name: string]: string }
}

type MetaOverlay = {
  capture_base: string,
  d: string,
  type: string,
  language: string,
  name: string,
  description: string
}

type UnitOverlay = {
  capture_base: string,
  d: string,
  type: string,
  metric_system: string,
  attribute_units: { [attribute_name: string]: string }
}

type StandardOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attribute_standards: { [attribute_name: string]: string }
}

type SubsetOverlay = {
  capture_base: string,
  d: string,
  type: string,
  attributes: string[]
}


/**
*/
export class Attribute {
  free(): void;
/**
* @param {string | IAttribute} name_or_object
*/
  constructor(name_or_object: string | IAttribute);
/**
* @param {any} attr_type
* @returns {Attribute}
*/
  setAttributeType(attr_type: any): Attribute;
/**
* @returns {Attribute}
*/
  setFlagged(): Attribute;
/**
* @param {Attribute} attr
* @returns {Attribute}
*/
  merge(attr: Attribute): Attribute;
/**
* @param {Encoding} encoding
* @returns {Attribute}
*/
  setEncoding(encoding: Encoding): Attribute;
/**
* @param {string} cardinality
* @returns {Attribute}
*/
  setCardinality(cardinality: string): Attribute;
/**
* @param {string} condition
* @returns {Attribute}
*/
  setCondition(condition: string): Attribute;
/**
* @param {any} data
* @returns {boolean}
*/
  checkCondition(data: any): boolean;
/**
* @param {'O' | 'M'} conformance
* @returns {Attribute}
*/
  setConformance(conformance: 'O' | 'M'): Attribute;
/**
* @param {string} format
* @returns {Attribute}
*/
  setFormat(format: string): Attribute;
/**
* @param {{ [language: string]: string }} labels
* @returns {Attribute}
*/
  setLabel(labels: { [language: string]: string }): Attribute;
/**
* @param {{ [language: string]: string }} information
* @returns {Attribute}
*/
  setInformation(information: { [language: string]: string }): Attribute;
/**
* @param {{ [code: string]: { [language: string]: string } }} entries
* @returns {Attribute}
*/
  setEntries(entries: { [code: string]: { [language: string]: string } }): Attribute;
}
/**
*/
export class OCABox {
  free(): void;
/**
*/
  constructor();
/**
* @param {OCABundle} oca_bundle
* @returns {OCABox}
*/
  load(oca_bundle: OCABundle): OCABox;
/**
* @returns {IAttribute[]}
*/
  attributes(): IAttribute[];
/**
* @returns {IMeta}
*/
  meta(): IMeta;
/**
* @returns {string | undefined}
*/
  classification(): string | undefined;
/**
* @returns {AST}
*/
  toAST(): AST;
/**
* @param {string} classification
* @returns {OCABox}
*/
  addClassification(classification: string): OCABox;
/**
* @param {string} layout
* @returns {OCABox}
*/
  addFormLayout(layout: string): OCABox;
/**
* @param {string} layout
* @returns {OCABox}
*/
  addCredentialLayout(layout: string): OCABox;
/**
* @param {string} name
* @param {{ [language: string]: string }} values
* @returns {OCABox}
*/
  addMeta(name: string, values: { [language: string]: string }): OCABox;
/**
* @param {Attribute} attr
* @returns {OCABox}
*/
  addAttribute(attr: Attribute): OCABox;
/**
* @returns {OCABundle}
*/
  generateBundle(): OCABundle;
}
/**
*/
export class Validator {
  free(): void;
/**
*/
  constructor();
/**
* @param {any} languages
* @returns {Validator}
*/
  enforceTranslations(languages: any): Validator;
/**
* @param {OCABundle} oca_bundle
* @returns {any}
*/
  validate(oca_bundle: OCABundle): any;
}
